
close all;

//----------------------------------------
//		Variables, Parameters & Shocks
//----------------------------------------
    
// Endogenous variables
var c y r q s pie pih pif lop a ys pis rs er e_a e_g e_p e_c es_a es_p e_cp y_obs pif_obs q_obs r_obs ys_obs pis_obs rs_obs s_obs;
% r = i
% lop = psif
% ys, pis, rs = ystar, pistar, istar; 

// Exogenous variables
varexo 
epsa % Tech shock
epsm % MP shock
epsg % Pref shock
epsp % Risk-premium / UIP shock
epsc % Cost-push shock
epscp
epsas % Foreign tech shock
epsps % Foreign cost-push shock
epsms % Foreign interest rate
;

// Parameters
parameters alf bet delh delf teth tetf eta psi sig h chi
psir psip psiy psid psie
rhoa rhog rhop rhoc roas rops rors
sda sdm sdg sdp sdc sdps sdas sdms    sdcp rhocp
psirs psips psiys psids
sigs hs delhs teths psis
;

// Calibrations
alf  = 0.28; % calibrated, see Table I, JP10
bet  = 0.99; % calibrated, see Table I, JP10
chi  = 0.01; % calibrated, see Table I, JP10
sig  = 1.2; 
psi  = 1.5; % Inverse Frisch elasticity
teth = 0.5;
tetf = 0.5;
eta  = 1.5;
h    = 0.5;
delh = 0.5;
delf = 0.5;
psir = 0.5;
psip = 1.5;
psiy = 0.25;
psid = 0.25;
psirs = 0.5;
psips = 1.5;
psiys = 0.25;
psids = 0.25;
//psiy = 0;
//psid = 0;

psie = 0.25;
rhoa = 0.8;
rhog = 0.8;
rhop = 0; %0.8;
rhoc = 0.5;
rhocp = 0.5;
//foreign
sigs  = 1.2;
teths = 0.5; 
hs = 0.5; 
delhs = 0.5; 
psis  = 1.5;


//standard deviation shocks
sdps = 0.5; 
sdas = 0.5;
sdms = 0.5;

sda = 0.5;
sdm = 0.5;
sdg = 0.5;
sdp = 0.5;
sdc = 0.5;
sdcp = 0.5;

roas = 0.8;
rops = 0.5;
rors = 0.8;

//----------------------------------------
//		Model
//----------------------------------------

model(linear);
    
   
// 14 Euler equation 
c - h*c(-1) = (c(+1)-h*c) - (1-h)/sig*(r - pie(+1) - e_g + e_g(+1));

// 15 Goods market clearing
(1-alf)*c = y - alf*eta*(2-alf)*s - alf*eta*lop - alf*ys;

// 16 Terms of trade
s - s(-1) = pif - pih + e_cp;

// 17 Real exchange rate
q = lop + (1-alf)*s;

// 18 Firms' optimal prices
pih - delh*pih(-1) = (1-teth)/teth*(1-bet*teth)*( psi*y - (1+psi)*e_a + alf*s + (1-h)/sig*(c-h*c(-1)) ) + bet*(pih(+1)-delh*pih);

// 19 Retailers' optimal prices
pif - delf*pif(-1) = (1-tetf)/tetf*(1-bet*tetf)*lop + bet*(pif(+1)-delf*pif) + e_c;

// 20 CPI identity
pie = pih + alf*(s-s(-1));

// 21 Uncovered interest rate parity
(r - pie(+1)) - (rs - pis(+1)) = (q(+1)-q) - chi*a - e_p;

// 22 Flow budget constraint
c + a = a(-1)/bet - alf*(s+lop) + y;

// 23 Taylor rule
r = psir*r(-1) + psip*pie + psiy*y + psid*(y-y(-1)) + epsm;

// 27 Nominal exchange rate (identity!)
er = er(-1) - (q-q(-1)) + pis - pie;

e_a = rhoa*e_a(-1) + epsa;
e_g = rhog*e_g(-1) + epsg; 
e_p = rhop*e_p(-1) + epsp;
e_c = rhoc*e_c(-1) + epsc;
e_cp = rhocp*e_cp(-1) + epscp;


// foreign output
ys - hs*ys(-1) = ys(+1) - hs*ys - (1-hs)/sigs*(rs - pis(+1)); 

//foreign inflation
pis - delhs*pis(-1) = (1-teths)/teths*(1-bet*teths)*( psis*ys - (1+psis)*es_a + (1-hs)/sigs*(ys-hs*ys(-1)) ) + bet*(pis(+1)-delhs*pis) + es_p;

//foreign Taylor rule
rs = psirs*rs(-1) + psips*pis + psiys*ys + psids*(ys-ys(-1)) + epsms; 

es_a = roas*e_a(-1) + epsas;
es_p = rops*e_p(-1) + epsps;


                    // 24 AR(1) for ys
                    //ys = roys*ys(-1) + sgys*epys;

                    // 25 AR(1) for pis
                    //pis = rops*pis(-1) + sgps*epps;

                    // 26 AR(1) for rs
                    //rs = rors*rs(-1) + sgrs*eprs;

y_obs = y - steady_state(y);
pif_obs = pif - steady_state(pif);
r_obs = r - steady_state(r);
q_obs = q - steady_state(q);
s_obs = s - steady_state(s);
ys_obs = ys - steady_state(ys);
rs_obs = rs -steady_state(rs);
pis_obs = pis - steady_state(pis);



end;

initval;
e_a = 0;
e_g = 0;
e_p = 0;
e_c = 0;

es_a = 0;
es_p = 0;
end;

steady;
check;

//----------------------------------------
//		Shocks
//----------------------------------------

//shocks;

//var epsa;
//stderr = 0.5; %sda^2;
//var epsm = stderr; %sdm^2;
//var epsg = stderr; %sdg^2;
//var epsp = stderr; %sdp^2;
//var epsc = stderr; %sdc^2;
//                          var epscp = stderr; %sdcp^2;
//var epsas = stderr; %sdas^2;
//var epsps = stderr; %sdps^2;
//var epsms = stderr; %sdms^2;

//end;

//----------------------------------------
//		Observed variables
//----------------------------------------


//observed model variables (names must coincide with data variables in datafile)
varobs y_obs pif_obs q_obs r_obs ys_obs pis_obs rs_obs s_obs;

//----------------------------------------
//		Parameter to be estimated
//----------------------------------------

//specify parameter to be estimated, including their priors


estimated_params;
sig, gamma_pdf, 1.2, 0.4;
psi, gamma_pdf, 1.5, 0.75;
teth, beta_pdf, 0.5, 0.1;
tetf,,beta_pdf, 0.5, 0.1;
eta, gamma_pdf, 1.5, 0.75;
h, beta_pdf, 0.5, 0.1;
delh, beta_pdf, 0.5, 0.1;
delf, beta_pdf, 0.5, 0.1;
psir, beta_pdf, 0.5, 0.1;
psip, gamma_pdf, 1.5, 0.3;
psiy, gamma_pdf, 0.25, 0.13;
psid, gamma_pdf, 1.5, 0.3;
psirs, beta_pdf, 0.5, 0.1;
psips, gamma_pdf, 1.5, 0.3;
psiys, gamma_pdf, 0.25, 0.13;
psids, gamma_pdf, 1.5, 0.3;
//psiy = 0;
//psid = 0;

//foreign

sigs, gamma_pdf, 1.2, 0.4;
psis, gamma_pdf, 1.5, 0.75;
teths, beta_pdf, 0.5, 0.1;
hs, beta_pdf, 0.5, 0.1;
delhs, beta_pdf, 0.5, 0.1;

//psie = 0.25;
rhoa, beta_pdf, 0.8, 0.1;
rhog, beta_pdf, 0.8, 0.1;
//rhop, beta_pdf, 0.8, 0.1;
rhoc, beta_pdf, 0.5, 0.25;
//rhocp, beta_pdf, 0.5, 0.25;


roas, beta_pdf, 0.8, 0.1;
rops, beta_pdf, 0.5, 0.25;
//rors, beta_pdf, 0.5, 0.25;
//standard deviation shocks
stderr epsa, inv_gamma_pdf, 0.5, inf; 
stderr epsg, inv_gamma_pdf, 0.5, inf; 
stderr epsp, inv_gamma_pdf, 0.5, inf; 
stderr epsc, inv_gamma_pdf, 0.5, inf; 
stderr epsm, inv_gamma_pdf, 0.5, inf;

stderr epsas, inv_gamma_pdf, 0.5, inf; 
stderr epsps, inv_gamma_pdf, 0.5, inf; 
stderr epsms, inv_gamma_pdf, 0.5, inf;

end;


//----------------------------------------
//		Computation
//----------------------------------------

//estimate model mode_file = code_mode, 
estimation(mode_compute=6, lik_init=1,plot_priors=1,datafile=data_soe_o,mh_replic=20000,mh_nblocks=2,mh_drop=0.5,mh_jscale=1,mode_check); 
//moments_varendo mode_compute=0, mode_file = JP10_est_mode
//stochastic simulation using the estimated parameters
stoch_simul(irf=40,order=1,hp_filter=1600);

//stoch_simul(irf=25) c y r s pie lop a pif er;

